package com.xkrs.utils;

import com.xkrs.dao.FirePointDao;
import com.xkrs.model.entity.FirePointEntity;
import org.apache.hc.core5.util.TextUtils;
import org.springframework.data.domain.Sort;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import javax.persistence.criteria.Predicate;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

/**
 * 火点查询工具类
 */
@Component
public class FirePointQueryHelper {

    @Resource
    private FirePointDao firePointDao;

    /**
     * 动态多条件查询火点列表
     *
     * @param code          区划编码，可以是省市区县街道任意编码，如果传需要至少两位。不传代表查询全国火点
     * @param startTime     过滤火点开始卫星时间，不传代表不指定开始时间。后台会有限制。
     * @param endTime       过滤火点结束卫星时间，不传代表不指定结束时间。
     * @param satelliteType 过滤传感器类型，不传代表查询全部类型的传感器
     * @param landType      过滤地物类型，不传代表查询全部地物类型
     * @return 符合条件的火点列表
     */
    public List<FirePointEntity> queryFirePoint(String code, String startTime, String endTime, String satelliteType, String landType) {
        Specification<FirePointEntity> specification = (root, criteriaQuery, criteriaBuilder) -> {
            //查询条件集合
            List<Predicate> predicateList = new ArrayList<>();
            //添加区划编码查询条件
            if (!TextUtils.isEmpty(code)) {
                try {
                    String formatCutCode = FirePointCodeUtils.getFormatCutCode(code);
                    if (!TextUtils.isEmpty(formatCutCode)) {
                        predicateList.add(criteriaBuilder.like(root.get("streetCode").as(String.class), formatCutCode + "%"));
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
            //限制最早开始时间不得早于一个月
            LocalDateTime lastMonth1 = LocalDateTime.now().minusDays(30);
            LocalDateTime lastMonth2 = LocalDateTime.of(lastMonth1.getYear(), lastMonth1.getMonth(), lastMonth1.getDayOfMonth(), 0, 0, 0, 0);
            String lastMonthString = DateTimeUtils.localDateTimeToString(lastMonth2);
//            predicateList.add(criteriaBuilder.greaterThanOrEqualTo(root.get("satelliteTime").as(String.class), lastMonthString));
            //开始时间查询条件
            if (!TextUtils.isEmpty(startTime)) {
                LocalDateTime startTime1 = DateTimeUtils.stringToLocalDateTime(startTime);
                LocalDateTime startTime2 = LocalDateTime.of(startTime1.getYear(), startTime1.getMonth(), startTime1.getDayOfMonth(), 0, 0, 0, 0);
                String startTimeString = DateTimeUtils.localDateTimeToString(startTime2);
                predicateList.add(criteriaBuilder.greaterThanOrEqualTo(root.get("satelliteTime").as(String.class), startTimeString));
            }
            //结束时间查询条件
            if (!TextUtils.isEmpty(endTime)) {
                LocalDateTime endTime1 = DateTimeUtils.stringToLocalDateTime(endTime).plusDays(1);
                LocalDateTime endTime2 = LocalDateTime.of(endTime1.getYear(), endTime1.getMonth(), endTime1.getDayOfMonth(), 0, 0, 0, 0);
                String endTimeString = DateTimeUtils.localDateTimeToString(endTime2);
                predicateList.add(criteriaBuilder.lessThanOrEqualTo(root.get("satelliteTime").as(String.class), endTimeString));
            }
            //卫星类型查询条件
            if (!TextUtils.isEmpty(satelliteType)) {
                predicateList.add(criteriaBuilder.equal(root.get("satelliteType").as(String.class), satelliteType));
            }
            //地物类型查询条件
            if (!TextUtils.isEmpty(landType)) {
                predicateList.add(criteriaBuilder.equal(root.get("landType").as(String.class), landType));
            }
            Predicate[] predicateArray = new Predicate[predicateList.size()];
            return criteriaBuilder.and(predicateList.toArray(predicateArray));
        };
        return firePointDao.findAll(specification, Sort.by(Sort.Direction.DESC, "satelliteTime"));
    }

    public List<FirePointEntity> queryZZHD(String code, String startTime, String endTime) {
        Specification<FirePointEntity> specification = (root, criteriaQuery, criteriaBuilder) -> {
            //查询条件集合
            List<Predicate> predicateList = new ArrayList<>();
            predicateList.add(criteriaBuilder.equal(root.get("countyCode").as(String.class), code));
            //开始时间查询条件
            if (!TextUtils.isEmpty(startTime)) {
                LocalDateTime startTime1 = DateTimeUtils.stringToLocalDateTime(startTime);
                LocalDateTime startTime2 = LocalDateTime.of(startTime1.getYear(), startTime1.getMonth(), startTime1.getDayOfMonth(), 0, 0, 0, 0);
                String startTimeString = DateTimeUtils.localDateTimeToString(startTime2);
                predicateList.add(criteriaBuilder.greaterThanOrEqualTo(root.get("satelliteTime").as(String.class), startTimeString));
            }
            //结束时间查询条件
            if (!TextUtils.isEmpty(endTime)) {
                LocalDateTime endTime1 = DateTimeUtils.stringToLocalDateTime(endTime).plusDays(1);
                LocalDateTime endTime2 = LocalDateTime.of(endTime1.getYear(), endTime1.getMonth(), endTime1.getDayOfMonth(), 0, 0, 0, 0);
                String endTimeString = DateTimeUtils.localDateTimeToString(endTime2);
                predicateList.add(criteriaBuilder.lessThanOrEqualTo(root.get("satelliteTime").as(String.class), endTimeString));
            }
            Predicate[] predicateArray = new Predicate[predicateList.size()];
            return criteriaBuilder.and(predicateList.toArray(predicateArray));
        };
        return firePointDao.findAll(specification, Sort.by(Sort.Direction.DESC, "satelliteTime"));
    }

}